import * as React from 'react';
import useNavigation from './useNavigation';
import { generatePath } from './navigationUtils';
import useModal from './useModal';
import RidgeNavigationContext from './contexts/RidgeNavigationContext';
import useBottomTabIndex from './useBottomTabIndex';
function isModifiedEvent(event) {
  return !!(event.metaKey || event.altKey || event.ctrlKey || event.shiftKey);
}
const staleTimeInSeconds = 5;
function isStale(lastPreloadedAt) {
  if (!lastPreloadedAt) {
    return true;
  }
  if (Date.now() - lastPreloadedAt > staleTimeInSeconds * 1000) {
    return true;
  }
  return false;
}
export default function Link(_ref) {
  let {
    to,
    toBottomTab,
    params,
    children,
    linkMode = 'default',
    onPress: onCustomPress,
    skipLinkBehaviourIfPressIsDefined,
    replace: isReplaceInsteadOfPush,
    refresh: isRefreshInsteadOfPush,
    ...rest
  } = _ref;
  const {
    basePath
  } = React.useContext(RidgeNavigationContext);
  const {
    currentTab
  } = useBottomTabIndex();
  const {
    inModal
  } = useModal();
  const {
    push,
    replace,
    refresh,
    preload,
    preloadElement,
    currentRootKey
  } = useNavigation();
  let path = to.path;
  if (toBottomTab !== null && toBottomTab !== void 0 && toBottomTab.path) {
    path = toBottomTab.path;
  } else if (currentTab !== null && currentTab !== void 0 && currentTab.path) {
    path = currentTab.path + path;
  }
  let href = generatePath('/' + currentRootKey + path, params);
  if (basePath) {
    href = '/' + basePath + href;
  }
  const lastPreloadedAt = React.useRef(null);
  React.useEffect(() => {
    lastPreloadedAt.current = null;
  }, [href]);
  const preloadData = React.useCallback(() => {
    if (!isStale(lastPreloadedAt.current)) {
      return;
    }
    lastPreloadedAt.current = new Date().getTime();
    preload(to, params);
  }, [preload, to, params]);
  const preloadDataAndElement = React.useCallback(() => {
    preloadElement(to);
    preloadData();
  }, [preloadElement, to, preloadData]);
  const onPress = React.useCallback(event => {
    // we don't want to go to another screen but we do want preloading
    // behaviour of the Link component :)
    // e.g. a modal with same data dependencies as the list screen
    if (onCustomPress) {
      onCustomPress(event);
      return;
    }
    const nativeEvent = event.nativeEvent;
    if (!event.defaultPrevented &&
    // onClick prevented default
    nativeEvent.button === 0 &&
    // Ignore everything but left clicks
    // (!target || target === '_self') && // Let browser handle "target=_blank" etc.
    !isModifiedEvent(nativeEvent) // Ignore clicks with modifier keys
    ) {
      event.preventDefault();
      const options = {
        preload: isStale(lastPreloadedAt.current),
        toBottomTab
      };
      if (isRefreshInsteadOfPush) {
        refresh(to, params, options);
      } else if (isReplaceInsteadOfPush) {
        replace(to, params, options);
      } else {
        push(to, params, options);
      }
    }
  }, [onCustomPress, toBottomTab, isRefreshInsteadOfPush, isReplaceInsteadOfPush, refresh, to, params, replace, push]);
  const preloadElementInner = React.useCallback(() => {
    preloadElement(to);
  }, [preloadElement, to]);
  const onPressInExternal = rest === null || rest === void 0 ? void 0 : rest.onPressIn;
  const loadDataOnHover = linkMode === 'sensitive';
  const onPressIn = React.useCallback(e => {
    if (!loadDataOnHover) {
      preloadData();
    }
    onPressInExternal === null || onPressInExternal === void 0 ? void 0 : onPressInExternal(e);
  }, [loadDataOnHover, onPressInExternal, preloadData]);
  const onHoverInExternal = rest === null || rest === void 0 ? void 0 : rest.onHoverIn;
  const onHoverIn = React.useCallback(e => {
    if (loadDataOnHover) {
      preloadDataAndElement();
    } else {
      preloadElementInner();
    }
    onHoverInExternal === null || onHoverInExternal === void 0 ? void 0 : onHoverInExternal(e);
  }, [loadDataOnHover, onHoverInExternal, preloadDataAndElement, preloadElementInner]);
  let baseProps = {
    onPressIn: onPressIn,
    onHoverIn: onHoverIn,
    onPress: onPress
  };
  let childrenProps = onCustomPress || inModal ? baseProps : {
    ...baseProps,
    accessibilityRole: 'link',
    href
  };
  if (skipLinkBehaviourIfPressIsDefined && onCustomPress) {
    children({
      onPress: onCustomPress
    });
  }
  return children(childrenProps);
}
//# sourceMappingURL=Link.web.js.map