import * as React from 'react';
import { StateNavigator } from 'navigation';
import { generatePath, getPathFromUrl, getRootKeyFromPath, getScreenKey, makeVariablesNavigationFriendly, rootKeyAndPaths } from './navigationUtils';
import { defaultTheme } from './theme';
import { Platform, useColorScheme } from 'react-native';
import { NavigationHandler } from 'navigation-react';
import NavigationStack from './navigation/NavigationStack';
import BottomTabsStack from './BottomTabsStack';
import RidgeNavigationContext from './contexts/RidgeNavigationContext';
import NavigationStackWrapper from './NavigationStackWrapper';
import { OptimizedContextProvider } from './contexts/OptimizedContext';
import getHistoryManager from './navigation/historyManager';
import BottomTabBadgeProvider from './contexts/BottomTabBadgeProvider';
import BottomTabIndexProvider from './contexts/BottomTabIndexProvider';
import HiddenNavbarWithSwipeBack from './HiddenNavbarWithSwipeBack';
import BottomTabRefreshProvider from './contexts/BottomTabRefreshProvider';
import { useNavigationUrl } from './useNavigationUrl';
export default function NavigationProvider(_ref) {
  let {
    screens,
    navigationRoot,
    SuspenseContainer,
    themeSettings,
    children,
    initialRootKey,
    basePath
  } = _ref;
  const {
    setGoToUrl,
    goToUrl,
    initialDefaultUrl
  } = useNavigationUrl({
    navigationRoot,
    initialRootKey,
    basePath
  });
  const colorScheme = useColorScheme();
  const theme = React.useMemo(() => (themeSettings || defaultTheme)[colorScheme || 'light'], [colorScheme, themeSettings]);
  const preloadedCache = React.useRef({});
  const preloadScreen = React.useCallback((screen, params) => {
    try {
      const result = screen.preload(params);
      const path = generatePath(screen.path, params);
      preloadedCache.current[path] = result;
    } catch (error) {
      console.log('[react-native-ridge-navigation] error while preloading screen', screen, params, error);
    }
  }, []);
  const preloadElement = React.useCallback(async screen => {
    try {
      var _screen$element, _screen$element$prelo;
      (_screen$element = screen.element) === null || _screen$element === void 0 ? void 0 : (_screen$element$prelo = _screen$element.preload) === null || _screen$element$prelo === void 0 ? void 0 : _screen$element$prelo.call(_screen$element);
    } catch (error) {
      console.log('[react-native-ridge-navigation] error while preloading element', screen, error);
    }
  }, []);
  const preloadRoot = React.useCallback(rootKey => {
    const root = navigationRoot[rootKey];
    if (!root) {
      console.log('[react-native-ridge-navigation] No root found for key', rootKey);
      return;
    }
    switch (root.type) {
      case 'bottomTabs':
        root.children.forEach(tab => {
          preloadElement(tab.child);
          preloadScreen(tab.child, {});
        });
        break;
      case 'normal':
        if (root.child) {
          preloadElement(root.child);
          preloadScreen(root.child, {});
        }
    }
  }, [navigationRoot, preloadElement, preloadScreen]);
  const rootNavigator = React.useMemo(() => {
    const navigators = Object.entries(navigationRoot).map(_ref2 => {
      let [rootKey, root] = _ref2;
      switch (root.type) {
        case 'bottomTabs':
          // on the web we don't have a nested stack because we want to preserve the history
          // on native, we need to nest the stack for bottom tabs
          return [{
            key: rootKey,
            route: rootKey,
            trackCrumbTrail: false,
            preload: () => preloadRoot(rootKey),
            preloadId: rootKey,
            renderScene: () => Platform.OS === 'web' ? null : /*#__PURE__*/React.createElement(BottomTabRefreshProvider, null, /*#__PURE__*/React.createElement(BottomTabIndexProvider, null, /*#__PURE__*/React.createElement(BottomTabsStack, null)))
          }, ...root.children.map(tab => {
            return screens.map(screen => {
              // we don't want the child path segment in the url
              const isTheSame = tab.child.path === screen.path;
              return {
                key: getScreenKey(rootKey, tab, screen.path),
                route: makeVariablesNavigationFriendly(getScreenKey(rootKey, tab, screen.path)),
                renderScene: () => /*#__PURE__*/React.createElement(screen.element, null),
                trackCrumbTrail: !isTheSame,
                screen,
                preloadId: screen.path
              };
            });
          }).flat(2)];
        case 'normal':
          return [{
            key: rootKey,
            route: rootKey,
            trackCrumbTrail: false,
            renderScene: () => /*#__PURE__*/React.createElement(root.child.element, null),
            screen: root.child,
            preload: () => preloadRoot(rootKey),
            preloadId: root.child.path
          }, ...screens.map(screen => ({
            key: rootKeyAndPaths(rootKey, screen.path),
            route: makeVariablesNavigationFriendly(rootKeyAndPaths(rootKey, screen.path)),
            renderScene: () => /*#__PURE__*/React.createElement(screen.element, null),
            trackCrumbTrail: true,
            screen,
            preloadId: screen.path
          }))];
      }
    }).flat();
    return new StateNavigator(navigators, getHistoryManager(basePath));
  }, [basePath, navigationRoot, preloadRoot, screens]);
  const preloadLink = React.useCallback(url => {
    var _state$preload;
    const {
      state,
      data,
      crumbs
    } = rootNavigator.parseLink(url);
    state === null || state === void 0 ? void 0 : (_state$preload = state.preload) === null || _state$preload === void 0 ? void 0 : _state$preload.call(state);
    if (state.screen) {
      preloadElement(state.screen);
      preloadScreen(state.screen, data);
    }
    crumbs.forEach(crumb => {
      var _crumb$state, _crumb$state$preload;
      (_crumb$state = crumb.state) === null || _crumb$state === void 0 ? void 0 : (_crumb$state$preload = _crumb$state.preload) === null || _crumb$state$preload === void 0 ? void 0 : _crumb$state$preload.call(_crumb$state);
      if (crumb.state.screen) {
        preloadElement(crumb.state.screen);
        preloadScreen(crumb.state.screen, crumb.data);
      }
    });
  }, [preloadElement, preloadScreen, rootNavigator]);
  const openUrl = React.useCallback((uri, change, fallback) => {
    try {
      const path = getPathFromUrl(uri);
      const rootKey = getRootKeyFromPath(path);
      const root = navigationRoot[rootKey];
      const rootType = root === null || root === void 0 ? void 0 : root.type;
      const isBottomTabs = rootType === 'bottomTabs';
      const isNative = Platform.OS !== 'web';
      if (isNative && isBottomTabs) {
        preloadRoot(rootKey);
        preloadLink(uri);
        rootNavigator.start(rootKey);
      } else {
        preloadLink(uri);
        rootNavigator.navigateLink(uri);
      }
    } catch (e) {
      console.log(`[react-native-ridge-navigation] can't convert deep link to the right stack: ${uri}`, e, {
        fallback,
        change
      });
      if (!fallback && !change) {
        console.debug('[react-native-ridge-navigation] there is no app started yet, lets go to the default url}', {
          defaultUrl: initialDefaultUrl
        });
        // setTimeout(() => {
        openUrl(initialDefaultUrl, false, true);
        // }, 1000);
      }
    }
  },
  // eslint-disable-next-line react-hooks/exhaustive-deps
  []);
  const fluent = React.useCallback(function (fluentParams) {
    var _children;
    let root = navigationRoot[fluentParams.rootKey];
    const tab = fluentParams.tab || (root === null || root === void 0 ? void 0 : (_children = root.children) === null || _children === void 0 ? void 0 : _children[0]);
    let fluentNavigator = new StateNavigator(rootNavigator).fluent().navigate(root.type === 'bottomTabs' ? getScreenKey(fluentParams.rootKey, tab) : fluentParams.rootKey);
    for (var _len = arguments.length, fluentScreens = new Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
      fluentScreens[_key - 1] = arguments[_key];
    }
    fluentScreens.forEach(fluentScreen => {
      fluentNavigator = fluentNavigator.navigate(root.type === 'bottomTabs' ? getScreenKey(fluentParams.rootKey, tab, fluentScreen.screen.path) : rootKeyAndPaths(fluentParams.rootKey, fluentScreen.screen.path), fluentScreen.params);
    });
    const url = fluentNavigator.url;
    if (process.env.NODE_ENV === 'development') {
      console.log(`[react-native-ridge-navigation] fluent navigation to ${url}`);
    }
    setGoToUrl(url);
  }, [navigationRoot, rootNavigator, setGoToUrl]);
  const [_, setForceRerender] = React.useState(0);
  const firstTime = React.useRef(true);
  React.useEffect(() => {
    if (goToUrl) {
      if (firstTime.current) {
        openUrl(goToUrl, false);
        firstTime.current = false;
        setForceRerender(prev => prev + 1);
      } else {
        const timerId = setTimeout(() => {
          openUrl(goToUrl, true);
        }, 50);
        return () => {
          clearTimeout(timerId);
        };
      }
    }
    return undefined;
  }, [goToUrl, openUrl]);
  if (firstTime.current) {
    return null;
  }
  return /*#__PURE__*/React.createElement(RidgeNavigationContext.Provider, {
    value: {
      screens,
      rootNavigator,
      navigationRoot,
      preloadedCache: preloadedCache.current,
      preloadRoot,
      preloadScreen,
      theme,
      preloadElement,
      SuspenseContainer,
      fluent,
      goToUrl,
      basePath
    }
  }, /*#__PURE__*/React.createElement(BottomTabBadgeProvider, null, /*#__PURE__*/React.createElement(NavigationHandler, {
    stateNavigator: rootNavigator
  }, /*#__PURE__*/React.createElement(NavigationStackWrapper, null, children && /*#__PURE__*/React.createElement(OptimizedContextProvider, {
    state: null,
    data: undefined,
    withSuspenseContainer: false
  }, children), /*#__PURE__*/React.createElement(NavigationStack, {
    underlayColor: theme.layout.backgroundColor,
    backgroundColor: () => theme.layout.backgroundColor,
    unmountStyle: () => '',
    renderScene: (state, data) => {
      return /*#__PURE__*/React.createElement(React.Fragment, null, /*#__PURE__*/React.createElement(HiddenNavbarWithSwipeBack, null), /*#__PURE__*/React.createElement(OptimizedContextProvider, {
        state: state,
        data: data
      }, /*#__PURE__*/React.createElement(OptimizedRenderScene, {
        renderScene: state.renderScene
      })));
    }
  })))));
}
const OptimizedRenderScene = /*#__PURE__*/React.memo(_ref3 => {
  let {
    renderScene
  } = _ref3;
  return renderScene();
});

// fluent(root.HomeScreen).
// bottomTab(bottomRoots.Post).
// push().
// push().
// push();
//# sourceMappingURL=NavigationProvider.js.map