import * as React from 'react';
import { getScreenKey } from './navigationUtils';
import useCurrentRoot from './useCurrentRoot';
import OptimizedContext from './contexts/OptimizedContext';
import useBottomTabIndex from './useBottomTabIndex';
import { Platform } from 'react-native';
import RidgeNavigationContext from './contexts/RidgeNavigationContext';
export default function useNavigation() {
  const {
    rootNavigator,
    preloadRoot,
    preloadScreen,
    preloadElement,
    theme,
    stateNavigator
  } = React.useContext(OptimizedContext);
  const {
    fluent,
    navigationRoot
  } = React.useContext(RidgeNavigationContext);
  const {
    currentRootKey,
    currentRoot
  } = useCurrentRoot();
  const {
    currentTab,
    switchToTab
  } = useBottomTabIndex();
  const preload = React.useCallback(async (screen, params) => {
    preloadScreen(screen, params);
  }, [preloadScreen]);
  const canNavigateBack = React.useCallback(dist => {
    return stateNavigator.canNavigateBack(dist || 1);
  }, [stateNavigator]);
  const popToTop = React.useCallback(() => {
    const {
      crumbs
    } = stateNavigator.stateContext;
    stateNavigator.navigateBack(crumbs.length);
  }, [stateNavigator]);
  const pop = React.useCallback(dist => {
    let distance = 1;
    if (typeof dist === 'number') {
      distance = dist;
    }
    if (stateNavigator.canNavigateBack(distance)) {
      stateNavigator.navigateBack(distance);
    } else {
      console.log('can not navigate back');
    }
  }, [stateNavigator]);
  const switchRoot = React.useCallback(function (rootKey) {
    let preloadSetting = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : true;
    if (preloadSetting) {
      preloadRoot(rootKey);
    }
    const root = navigationRoot[rootKey];
    const screenKey = Platform.select({
      web: getScreenKey(rootKey, (root === null || root === void 0 ? void 0 : root.type) === 'bottomTabs' ? root.children[0] : undefined, (root === null || root === void 0 ? void 0 : root.type) === 'bottomTabs' ? root.children[0].path : root === null || root === void 0 ? void 0 : root.child.path),
      default: rootKey
    });
    rootNavigator.start(screenKey);
  }, [preloadRoot, navigationRoot, rootNavigator]);
  const refresh = React.useCallback((screen, params, options) => {
    if (options !== null && options !== void 0 && options.preload) {
      preload(screen, params);
    }
    stateNavigator.refresh(params, 'replace');
  }, [stateNavigator, preload]);
  const innerNavigate = React.useCallback((screen, params, options, historyAction) => {
    if (options !== null && options !== void 0 && options.preload) {
      preload(screen, params);
    }

    // on web, it works based on url but on mobile it works based on bottom tab index
    if (options !== null && options !== void 0 && options.toBottomTab && Platform.OS !== 'web') {
      switchToTab(options.toBottomTab);
    } else {
      const screenKey = getScreenKey(currentRootKey, (options === null || options === void 0 ? void 0 : options.toBottomTab) || currentTab, screen.path);
      stateNavigator.navigate(screenKey, params, historyAction);
    }
  }, [currentRootKey, currentTab, stateNavigator, preload, switchToTab]);
  const push = React.useCallback((screen, params, options) => {
    innerNavigate(screen, params, options);
  }, [innerNavigate]);
  const replace = React.useCallback((screen, params, options) => {
    if (options !== null && options !== void 0 && options.preload) {
      preload(screen, params);
    }
    const screenKey = getScreenKey(currentRootKey, (options === null || options === void 0 ? void 0 : options.toBottomTab) || currentTab, screen.path);
    const url = stateNavigator.fluent(true).navigateBack(1).navigate(screenKey, params).url;
    stateNavigator.navigateLink(url);
  }, [currentRootKey, currentTab, preload, stateNavigator]);
  return {
    currentRootKey,
    currentRoot,
    preloadElement,
    preloadRoot,
    preload,
    pop,
    popToTop,
    switchRoot,
    push,
    replace,
    refresh,
    theme,
    canNavigateBack,
    fluent
  };
}
//# sourceMappingURL=useNavigation.js.map